#include <xc.h>
//이 Header 삽입문은 맨 처음 삽입해야 하는 삽입문으로 프로젝트에서 선택된
//PIC 마이크로컨트롤러의 헤더 파일로 연결되는 Header 삽입문이다.
//일반적으로는 #inlcude <pic16f1939.h> 로 삽입될 수도 있지만,
//PIC MCU가 변경시에는 모든 파일의 이름을 변경해야 하는 단점이 있다.

__CONFIG(FOSC_INTRC_CLKOUT & WDTE_OFF & PWRTE_ON & MCLRE_OFF & BOREN_OFF);
//위 설정은 내장된 오실레이터 16MHz를 사용하고, 와치독 타이머는 끄고,
//파워-업 타이머는 켜고, MCLR핀은 외부 리셋 기능 끄고,
//BOR 리셋 기능도 꺼진 상태로 설정된다.

//PICDEM 2 PLUS 보드를 사용시
#define PICDEM2PLUS
//#define INSEM_8bitPIC

//PIC MCU에서 사용하는 시스템 클럭 계산
#define Fosc 16000000 			//내부 16MHz 클럭 설정
#define Fcy Fosc / 4 			// Fcy = Fosc/4
#define Tcy 1 / Fcy 			// 1/4MHz = 250ns

// LED 출력을 위한 I/O 포트 선언문이다.
#ifdef PICDEM2PLUS
    #define  LED		PORTBbits.RB0 //D2 LED 출력, 비트별 설정 표현 가능
#endif
#ifdef INSEM_8bitPIC
    #define  LED		PORTDbits.RD0	//D1  LED 출력

#endif


#ifdef PICDEM2PLUS
   void initial_port(void)
   {
	//PORTB 사용 LED 출력 포트 초기화
	TRISB = 0b11111110; //PORTB RB0핀을 출력으로 설정
	// 1 = 입력, 0 = 출력
	//ANSELB = 0x00; //포트B의 ANxx 핀들 모두 디지털로 설정
   }
#endif

#ifdef INSEM_8bitPIC
   void initial_port(void)
   {
	//PORTD 사용 LED 출력 포트 초기화
	TRISD = 0b11111110; //PORTD RD0핀을 출력으로 설정
	// 1 = 입력, 0 = 출력
	ANSELD = 0x00; //포트D의 ANxx 핀들 모두 디지털로 설정
   }
#endif

void initial_timer0(void)
{
	TMR0 = 0; 	//타이머0 설정
	T0IE = 0; 	//타이머0 인터럽트 설정 : 인터럽트 기능 Disable
	T0IF = 0; 	//타이머0의 인터럽트 플래그 초기화 -> 0
	OPTION_REG = 0b00000111; 	//타이머0 프리스케일러 1:256
}


int main(int argc, char** argv)
{
    unsigned char tmr0_count; 	// 1초 인터럽트 회수 카운팅하는 변수

    //오실레이터 초기화
    OSCCON = 0b01111000;	//16MHZ 내부 클럭 설정

    initial_port();             //LED를 위한 포트 초기화 함수 실행

    initial_timer0();           //타이머0 초기화

    tmr0_count =0; 			//변수 초기화

    while(1)
    {
        if(T0IF == 1)	//TMR0 레지스터 카운트 값이 255에서 0으로 바뀔 때
                        //이 T0IF 타이머0 인터럽트 플래그 비트가 1로 바뀐다.
        {
            T0IF=0; 	//타이머0 인터럽트 플래그 다시 0을 만들어서 시간재기
            tmr0_count++;	//약61ms 마다 1씩 증가
            if(tmr0_count==61)	//변수 값이 61 즉, 1초가 됨
            {
                LED ^= 1; 	//LED Toggle
                tmr0_count = 0; //다시 1초 시간 재기 위해 61번 카운트하기 위해 리셋
            }
        }
    }
}
